﻿Type=StaticCode
Version=2.52
@EndOfDesignText@
'Code module
'Subs in this code module will be accessible from all modules.
Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
	Dim manager As UsbManager	
	Dim devices() As UsbDevice
	Dim interfaces(1) As UsbInterface
	Dim rawdescs() As Byte
End Sub

Sub Initialize
	manager.Initialize
End Sub

' Get all the device presently attached to the USB host port
Sub GetDevices
	devices = manager.GetDevices
End Sub

' Return the number of devices present.
Sub NumberOfDevices
	Return devices.Length
End Sub

' Check the application has permission to access USB, returns True if so
Sub CheckPermission(device As Int) As Boolean
	Return manager.HasPermission(devices(device))
End Sub

' Request that the application has permission to access US, returns true if granted
Sub RequestPermission(device As Int) As Boolean
	manager.RequestPermission(devices(device))
End Sub

Sub DeviceDetails(device As Int)
	Dim sb As StringBuilder	,datalen As Int, idx As Int,Obj1 As Reflector,ud As UsbDevice, connection As UsbDeviceConnection
	sb.Initialize 
	ud = devices(device)	
	connection = Connect(0, 0, interfaces)

	Obj1.Target = connection
	Obj1.Target = Obj1.GetField("connection")
	rawdescs = Obj1.RunMethod("getRawDescriptors")
	Dim buffer(rawdescs(7)) As Byte ' maximum packet size
	
	' Manufacturer string index at byte 14
	idx = rawdescs(14)
	datalen = connection.ControlTransfer(128, 6, 0x0300 + idx, 0, buffer, buffer.Length, 100)
	sb.Append("Manufacturer : ").Append(Bytes0ToString(buffer)).Append(CRLF)
	LogBytes("Manufacturer :", buffer)

	' Product string index at byte 15
	idx = rawdescs(15)
	datalen = connection.ControlTransfer(128, 6, 0x0300 + idx, 0, buffer, buffer.Length, 100)
	sb.Append("Product : ").Append(Bytes0ToString(buffer)).Append(CRLF)
	LogBytes("Product :", buffer)
	
	' Serial string index at byte 16
	idx = rawdescs(16)
	datalen = connection.ControlTransfer(128, 6, 0x0300 + idx, 0, buffer, buffer.Length, 100)
	sb.Append("Serial : ").Append(Bytes0ToString(buffer)).Append(CRLF)
	LogBytes("Serial :", buffer)

	sb.Append(CRLF)
	sb.Append("DeviceName : ").Append(ud.DeviceName).Append(CRLF)
	sb.Append("DeviceClass : ").Append(USB_Class(ud.DeviceClass)).Append(" " & ud.DeviceClass & CRLF)
	sb.Append("DeviceSubClass : ").Append(ud.DeviceSubclass).Append(CRLF)
	sb.Append("Device ID : ").Append(ToHex(ud.DeviceId)).Append(CRLF)
	sb.Append("ProductId : ").Append(ToHex(ud.ProductId)).Append(CRLF)
	sb.Append("VendorId  :").Append(ToHex(ud.VendorId)).Append(CRLF)
	sb.Append(CRLF)
	'Iterate over interfaces
	For i = 0 To ud.InterfaceCount - 1
		Dim ifce As UsbInterface,endpoint As UsbEndpoint,DoIT As Boolean = True
		'Try
			ifce = ud.GetInterface(i)
			sb.Append("  B4aInterfaceNumber : ").Append(i).Append(CRLF)
			sb.Append("  InterfaceClass : ").Append(USB_Class(ifce.InterfaceClass)).Append(CRLF)
			sb.Append("  InterfaceSubClass : ").Append(ifce.InterfaceSubclass).Append(CRLF)
			sb.Append("  InterfaceProtocol : ").Append(ifce.InterfaceProtocol).Append(CRLF)
			sb.Append(CRLF)
		'Catch
		'	DoIT=False
		'End Try
		'If DoIT Then
			For j = 0 To ifce.EndpointCount - 1
				'Try
					'If DoIT Then
						endpoint = ifce.GetEndpoint(j)
						'sb.Append("    EndpointAddress : ").Append(endpoint.Address).Append(CRLF) ' contains number and direction
						sb.Append("    EndpointNumber : ").Append(endpoint.EndpointNumber).Append(CRLF)
						sb.Append("    EndpointDirection : ").Append(USB_EndpointDirection(endpoint.Direction)).Append(CRLF)
						sb.Append("    EndpointType : ").Append(USB_EndpointType(endpoint.Type)).Append(" " & endpoint.Type & CRLF)
						sb.Append("    EndpointAttribute : ").Append(endpoint.Attributes).Append(CRLF)
						sb.Append("    EndpointInterval : ").Append(endpoint.Interval).Append(CRLF)
						sb.Append("    EndpointMaxPacketSize : ").Append(endpoint.MaxPacketSize).Append(CRLF)
						sb.Append(CRLF)
					'End If
				'Catch
				'	DoIT=False
				'End Try
			Next
		'End If
	Next
	Return sb.ToString
End Sub

Sub USB_EndpointDirection(direction As Int)
	If direction = 0 Then
		Return "Out"
	End If
	Return "In"
End Sub

Sub USB_EndpointType(iftype As Int)
	Select iftype
		Case 0:		Return "USB_ENDPOINT_XFER_CONTROL (control)"
		Case 1:		Return "USB_ENDPOINT_XFER_ISOC (isochronous )"
		Case 2:		Return "USB_ENDPOINT_XFER_BULK (bulk)"
		Case 3:		Return "USB_ENDPOINT_XFER_INT (interrupt)"
		Case Else:	Return "Unknown end point type " &  ToHex(iftype)
	End Select
End Sub


Sub USB_Class(class As Int)
	Select class
		Case 0:		Return "USB_CLASS_PER_INTERFACE (per-interface basis)"
		Case 1:		Return "USB_CLASS_AUDIO (audio)"
		Case 2:		Return "USB_CLASS_COM (communication device)"
		Case 3:		Return "USB_CLASS_COMM (human interface device)"
		Case 6:		Return "USB_CLASS_STILL_IMAGE (camera)"
		Case 5:		Return "USB_CLASS_PHYSICA (physical)"
		Case 7:		Return "USB_CLASS_PRINTER (printer)"
		Case 8:		Return "USB_CLASS_MASS_STORAGE (mass storage)"
		Case 9:		Return "USB_CLASS_HUB (hub)"
		Case 10:	Return "USB_CLASS_CDC_DATA(CDC device)"		
		Case 11:	Return "USB_CLASS_CSCID (smart card)"
		Case 13:	Return "USB_CLASS_CONTENT_SEC (content security)"
		Case 14:	Return "USB_CLASS_VIDEO (video)"
		Case 224:	Return "USB_CLASS_WIRELESS_CONTROLLER (wireless controller)"
		Case 239:	Return "USB_CLASS_MISC (miscellaneous"
		Case 254:	Return "USB_CLASS_APP_SPEC (application specific)"
		Case 255:	Return "USB_CLASS_VENDOR_SPEC (vendor specific)"
		Case Else:	Return "Unknown class " & ToHex(class)			
	End Select
End Sub

Sub ToHex(number As Int)
	Return Bit.ToHexString(number).ToUpperCase & "h"
End Sub

Sub Bytes0ToString(data()As Byte) As String
	'Various cheap USB devices I've tried produce rubbish for some string descriptors hence the checks
	Dim str As String,conv As ByteConverter, idx As Int
	If data(0) = 0 Then Return "not available"' zero length descriptor should be byte count
	If data(1) <> 3 Then Return "bad descriptor type " & data(1)' not a string descriptor should be 2
	str = conv.StringFromBytes(data,"UTF-16LE")
	idx = str.IndexOf(Chr(0))
	Return str.SubString2(1, idx)
End Sub

Sub StringToBytes0 (S As String) As Byte()
	S = S & Chr(0)
	Return S.GetBytes("UTF8")
End Sub

Sub LogBytes(prefix As String, bytes() As Byte)
	Dim msg As String
	msg = prefix
	For i = 0 To bytes.Length -1
		msg = msg & " " & bytes(i)
	Next
	Log(msg)
End Sub

#Region Data 
Sub Connect(device As Int, interfacenumber As Int, interface() As UsbInterface) As UsbDeviceConnection
	interface(0) = devices(device).GetInterface(interfacenumber)
	Return manager.OpenDevice(devices(device), interface(0), True)
End Sub

#End Region